/*	init.c

	Copyright 2008 Fred Cooke

	This file is part of the FreeEMS project.

	FreeEMS software is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	FreeEMS software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with any FreeEMS software.  If not, see <http://www.gnu.org/licenses/>.

	We ask that if you make any changes to this file you send them upstream to us at admin@diyefi.org

	Thank you for choosing FreeEMS to run your engine! */

#include "freeems.h"
#include "interrupts.h"
#include "init.h"

/* Set the Phase Locked Loop to our desired frequency (80MHz) and switch to using it for clock (40MHz bus speed) */ 
void init_pll(void)
{
	CLKSEL &= PLLSELOFF;	/* Switches to base external OSCCLK to ensure PLL is not being used (off out of reset, but not sure if the monitor turns it on before passing control or not) */
	PLLCTL &= PLLOFF;		/* Turn the PLL device off to adjust its speed (on by default out of reset) */
	REFDV = PLLDIVISOR;		/* 16MHz / (3 + 1) = 4MHz Bus frequency */
	SYNR = PLLMULTIPLIER;	/* 4MHz * (9 + 1) = 40MHz Bus frequency */
	PLLCTL |= PLLON;		/* Turn the PLL device back on again at 80MHz */

	while (!(CRGFLG & PLLLOCK)){
		/* Do nothing while we wait till the PLL loop locks onto the target frequency. */
		/* Target frequency is given by (2 * (crystal frequency / (REFDV + 1)) * (SYNR + 1)) */
		/* Bus frequency is half PLL frequency and given by ((crystal frequency / (REFDV + 1)) * (SYNR + 1)) */
	}

	CLKSEL = PLLSELON;		/* Switches to PLL clock for internal bus frequency */
	/* from MC9S12XDP512V2.pdf Section 2.4.1.1.2 page 101 Third paragraph */
	/* This takes a MAXIMUM of 4 OSCCLK clock cylces PLUS 4 PLL clock cycles */
	/* During this time ALL clocks freeze, and CPU activity ceases */
	/* For this reason there is no point in waiting for this to occur, we already are */
}

void init_ints(void)
{
	// set up interrupts for all things here
	
	// real time
	
	// input timer channels
	
	// output compare channels
	
	// port H for testing
	
	// irq and xirq for testing
}

void init_io(void)
{
	/* for now, hard code all stuff to be inputs as per Freescale documentation,	*/
	/* later what to do will be pulled from flash configuration such that all		*/
	/* things are setup at once, and not messed with thereafter. when the port		*/
	/* something uses is changed via the tuning interface, the confuration will be	*/
	/* done on the fly, and the value burned to flash such that next boot happens	*/
	/* correctly and current running devices are used in that way.					*/
	
	
	/* Turn off the digital input buffers on the ATD channels */
	ATD0DIEN = ZEROS; /* You are out of your mind if you waste this on digital Inputs */
	ATD1DIEN0 = ZEROS; /* You are out of your mind if you waste this on digital Inputs (NOT-bonded, can't use) */	
	ATD1DIEN1 = ZEROS; /* You are out of your mind if you waste this on digital Inputs */
	/* TODO Second half of ATD1 - can we disable this somehow */
	
	/* And configure them all for analog input */
	ATD0CTL2 = 0x80; /* Turns on the ADC block. */
	ATD0CTL5 = 0xB0; /* Sets justification to right, multiplex and scan all channels. */
	// TODO find out if this is the default (i suspect it is)
	// TODO look into sampling techniques
	
	/* Set up the PWM component */
	// TODO set this up
	
	/* Set up the SCI0 interface for basic comms here */
	// TODO fill this out
	
	/* Initialise the Data Direction Registers */
	/* To outputs based on the note at the end of chapter 1.2.2 of MC9S12XDP512V2.pdf */
	DDRA = ONES; /* GPIO (8) */
	DDRB = ONES; /* GPIO (8) */
	DDRE = 0xFC; /* 0b_1111_1100 : Clock and mode pins PE0,PE1 are input only pins, the rest are GPIO */
	DDRK = ONES; /* Only 0,1,2,3,4,5,7, NOT 6 (7) */
	DDRS = ONES; /* SCI0, SCI1, SPI0 (8) */
	DDRT = 0xFC; /* 0b_1111_1100 set ECT pins 0,1 to IC and 2:7 to OC (8) */
	DDRM = ONES; /* CAN 0 - 3 (8) */
	DDRP = ONES; /* PWM pins (8) */
	DDRH = ONES; /* SPI1, SPI2 (8) */
	DDRJ = ONES; /* Only 0,1,6,7 are brought out on the 112 pin chip (4) */
	/* Configure the non bonded pins to output to avoid current drain (112 pin package) */ 
	DDRC = ONES; /* NON-bonded external data bus pins */
	DDRD = ONES; /* NON-bonded external data bus pins */
	/* AD0DDR1 You are out of your mind if you waste this on digital Inputs */
	/* AD1DDR1 You are out of your mind if you waste this on digital Inputs */

	/* Initialise the state of pins configured as output */
	/* Initialise to high such that grounded things are all turned off by default. */
	PORTA = 0x40;
	PORTB = ZEROS;
	PORTE = 0x9F; /* 0b_1001_1111 : PE7 should be high PE5 and PE6 should be low, the rest high */
	PORTK = ZEROS;
	PORTS = ONES;
	PORTT = 0x03; // TODO change this to whatever is suitable for capture and compare for the 2 rpm/engine position inputs and the 6 injector outputs.
	PORTM = ONES;
	PORTP = ONES; // TODO hook these up to the adc channels such that you can vary the brightness of an led with a pot.
	PORTH = 0xAA;
	PORTJ = ONES;
	/* AD0PT1 You are out of your mind if you waste this on digital Inputs */
	/* AD1PT1 You are out of your mind if you waste this on digital Inputs */
	
	/* Initialise pwm channels with frequency, and initial duty */
	// TODO set up PWM here!
	
	/* Setup the timer module for period, and IC/OC etc */
	TIOS = 0xFC; /* 0b_0011_1111 0 and 1 are input capture, 2 through 7 are output compare */
	// TODO set up the timer stuff here.
	
	/* Turn off and on and configure all the modules in an explicit way */
	// TODO set up and turn off all modules (CAN,SCI,SPI,IIC,etc)
	
}

void init_pram(void)
{
	// load the pages of tuning data etc from flash to paged ram for fast access on the fly
}

void init(void){
	
	// TODO preconfigure the bare minimum such that init goes smoothly.
	ATOMIC_START(); /* disable ALL interrupts while we configure the board ready for use. */
	// TODO configure registers etc of the general operation of the chip
	init_pll();	/* Set up the PLL and use it */
	// TODO maybe some of it after PLL and some before
	
	init_ints(); /* setup the individual interrupts etc */
	init_io(); /* setup all the pins and modules to be in low power harmless states */
	init_pram(); /* load the blocks of data flash to the paged ram blocks for uber fast data lookup. */
	 
	/* Set user/feature/config up here! */
	// checksum the code base excluding the areas of flash used for tuning data OR, include the tuning data and write a checksum in with it each time. display a warning via tacho, shiftlights, etc if stuffed.
	// ???
	ATOMIC_END(); /* reenable any configured interrupts. */
}
